//---------------------------------------------------------------------------
// hlavovy soubor definic obsluhy serioveho portu
#ifndef CommH
#define CommH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <syncobjs.hpp>
//---------------------------------------------------------------------------
// blok definic serioveho portu
#define sMsgExtention    "(Error: %d)"
#define sOpenError       "Error accessing specified device (Error: %d)"
#define sInvalidHandle   "Invalid device handle, access denied (Error: %d)"
#define sPortAlreadyOpen "Port already assigned (open) (Error: %d)"
#define sPortNotOpen     "Port not open (Error: %d)"
#define sSetupCommErr    "Error initializing Read/Write Buffers (Error: %d)"
#define sUpdateDCBErr    "Error updating DataControlBlock (Error: %d)"
#define sCommTimeoutsErr "Error updating CommTimeouts (Error: %d)"
#define sEscFuncError    "EscapeCommFunction failure (Error: %d)"
#define sReadError       "Read error (Error: %d)"
#define sWriteError      "Write error (Error: %d)"
#define PurgeRead        PURGE_RXABORT + PURGE_RXCLEAR
#define PurgeWrite       PURGE_TXABORT + PURGE_TXCLEAR
#define PurgeReadWrite   PurgeRead + PurgeWrite

// definice typu obsluznuch procedur pro udalosti
typedef void __fastcall (__closure *TCommEvent)(TObject* Sender,
                                                DWORD Status);
typedef void __fastcall (__closure *TCommErrorEvent)(TObject* Sender,
                                                     int Errors);
typedef void __fastcall (__closure *TCommRxCharEvent)(TObject* Sender,
                                                      DWORD Count);
typedef void __fastcall (__closure *TCommStatEvent)(TObject* Sender,
                                                    DWORD Status,
                                                    COMSTAT ComStat);
// definice vyjmenovanych typu serioveho portu
enum TCommEventType { evBreak, evCts, evDsr, evError, evRing,
                      evRlsd, evRxChar, evRxFlag, evTxEmpty };
typedef Set <TCommEventType,evBreak,evTxEmpty> TCommEventTypes;
enum TBaudRate { br110,br300,br600,br1200,br2400,br4800,br9600,br14400,
                 br19200,br38400,br56000,br57600,br115200,br128000,br256000 };
enum TParity { paNone,paOdd,paEven,paMark,paSpace };
enum TStopBits { sb10,sb15,sb20 };
enum TDataBits { da4,da5,da6,da7,da8 };
enum TFlowControl { fcNone,fcCTS,fcDTR,fcSoftware,fcDefault };
enum TCommOption { coParityCheck,coDsrSensitivity,coIgnoreXOff,
                   coErrorChar, coNullStrip};
typedef Set <TCommOption,coParityCheck,coNullStrip > TCommOptions;
//---------------------------------------------------------------------------
// definice tridy objektu obsluzneho procesu serioveho portu
class TCommEventThread : public TThread
{
  private :
     HANDLE FCommHandle ;
     DWORD FEventMask ;
     TCommEvent FOnSignal;
  protected :
     virtual void __fastcall Execute( void );
     void __fastcall DoOnSignal( void );
  public :
     __fastcall TCommEventThread::TCommEventThread( HANDLE Handle,
                                               TCommEventTypes MonitorEvents );
     __property TCommEvent OnSignal = { read=FOnSignal,write=FOnSignal } ;
};
//---------------------------------------------------------------------------
// definice trid v objektu serioveho portu
class TThreadComm;

class TCommEventChars : public TPersistent
{
  private :
    TThreadComm *FOwner ;
    char FXOnChar;
    char FXOffChar;
    char FErrorChar;
    char FEofChar;
    char FEvtChar;
    void __fastcall SetEventChar( int Index,char Value );
  public :
        __fastcall TCommEventChars::TCommEventChars( TThreadComm *Owner );
    virtual void __fastcall Assign( TPersistent *Source );
  __published :
    __property char XOnChar = { index=1,read=FXOnChar,write=SetEventChar,
                                default=17 };
    __property char XOffChar = { index=2,read=FXOffChar,write=SetEventChar,
                                 default=19 };
    __property char ErrorChar = { index=3,read=FErrorChar,write=SetEventChar,
                                  default=0 };
    __property char EofChar = { index=4,read=FEofChar,write=SetEventChar,
                                default=0 };
    __property char EvtChar = { index=5,read=FEvtChar,write=SetEventChar,
                                default=0 };
} ;
//---------------------------------------------------------------------------
// trida vlastniho objektu serioveho portu
class TThreadComm : public TComponent
{
  private:
    HANDLE FHandle;
    DCB    FDCB;
    String FDeviceName;
    TSimpleEvent *FEvent;
    TCriticalSection *FCriticalSection;
    int FReadTimeout;
    int FWriteTimeout;
    WORD FReadBufSize;
    WORD FWriteBufSize;
    TCommEventTypes FMonitorEvents;
    TBaudRate FBaudRate;
    TParity FParity;
    TStopBits FStopBits;
    TDataBits FDataBits;
    bool FConnected ;
    OVERLAPPED WRITE_OS ;
    OVERLAPPED READ_OS ;
    TCommEventThread *FEventThread;
    TCommEventChars *FEventChars;
    TCommOptions FOptions;
    TFlowControl FFlowControl;
    TNotifyEvent FOnBreak;
    TNotifyEvent FOnCts;
    TNotifyEvent FOnDsr;
    TCommErrorEvent FOnError;
    TNotifyEvent FOnRing;
    TNotifyEvent FOnRlsd;
    TCommRxCharEvent FOnRxChar;
    TNotifyEvent FOnRxFlag;
    TNotifyEvent FOnTxEmpty;
    TCommStatEvent FCommStatEvent ;
    void __fastcall SetDeviceName( String Value );
    void __fastcall SetMonitorEvents( TCommEventTypes Value );
    void __fastcall SetReadBufSize( WORD Value );
    void __fastcall SetWriteBufSize( WORD Value );
    void __fastcall SetBaudRate( TBaudRate Value );
    void __fastcall SetParity( TParity Value );
    void __fastcall SetStopBits( TStopBits Value );
    void __fastcall SetDataBits( TDataBits Value );
    void __fastcall SetOptions( TCommOptions Value );
    void __fastcall SetFlowControl( TFlowControl Value );
    bool __fastcall GetModemState( int Index );
    bool __fastcall GetComState( int Index );
    void __fastcall Lock( void );
    void __fastcall Unlock( void );
    void __fastcall EscapeComm( int Flag );
    void __fastcall UpdateCommTimeouts( void );
protected:
    virtual void __fastcall CreateHandle( void );
    void __fastcall DestroyHandle( void );
    void __fastcall HandleCommEvent( TObject *Sender,DWORD Status );
public:
    __fastcall TThreadComm( TComponent* Owner );
    virtual __fastcall ~TThreadComm( void );
    void __fastcall Open( HANDLE ModemHandle );
    void __fastcall Close( void );
    bool __fastcall Enabled( void );
    int __fastcall Write( char Buf[],DWORD Count );
    int __fastcall Read( char Buf[],DWORD Count );
    int __fastcall InQueCount( void );
    int __fastcall OutQueCount( void );
    void __fastcall PurgeIn( void );
    void __fastcall PurgeOut( void );
// escape funkce serioveho portu
    void __fastcall SetDTRState( bool State );
    void __fastcall SetRTSState( bool State );
    void __fastcall SetBREAKState( bool State );
    void __fastcall SetXONState( bool State );
    void __fastcall UpdateDataControlBlock( void );
    __property bool Connected = { read=FConnected };
// priznaky statusu serioveho portu
    __property bool CTS = { index=1,read=GetModemState };
    __property bool DSR = { index=2,read=GetModemState };
    __property bool RING = { index=3,read=GetModemState };
    __property bool RLSD = { index=4,read=GetModemState };
    __property bool CtsHold = { index=1,read=GetComState };
    __property bool DsrHold = { index=2,read=GetComState };
    __property bool RlsdHold = { index=3,read=GetComState };
    __property bool XOffHold = { index=4,read=GetComState };
    __property bool XOffSent = { index=5,read=GetComState };
// identifikace serioveho portu
    __property HANDLE Handle = { read=FHandle };
// konfiguracni funkce serioveho portu
    __property String DeviceName = { read=FDeviceName,write=SetDeviceName };
    __property int ReadTimeout  = { read=FReadTimeout,write=FReadTimeout,
                                    default=1000 };
    __property int WriteTimeout = { read=FWriteTimeout,write=FWriteTimeout,
                                    default=1000 };
    __property WORD ReadBufSize  = { read=FReadBufSize,write=SetReadBufSize,
                                     default=4096 };
    __property WORD WriteBufSize = { read=FWriteBufSize,write=SetWriteBufSize,
                                     default=4096 };
    __property TCommEventTypes MonitorEvents = { read=FMonitorEvents,
                                                 write=SetMonitorEvents };
    __property TBaudRate BaudRate = { read=FBaudRate,write=SetBaudRate,
                                      default=br9600 };
    __property TParity Parity = { read=FParity,write=SetParity,
                                  default=paNone };
    __property TStopBits StopBits = { read=FStopBits,write=SetStopBits,
                                      default=sb10 };
    __property TDataBits DataBits = { read=FDataBits,write=SetDataBits,
                                      default=da8 };
    __property TCommEventChars* EventChars = { read=FEventChars };
    __property TCommOptions Options = { read=FOptions,write=SetOptions };
    __property TFlowControl FlowControl = { read=FFlowControl,
                                            write=SetFlowControl,
                                            default=fcDefault };
// obsluha udalosti serioveho portu
    __property TNotifyEvent OnBreak = { read=FOnBreak,write=FOnBreak };
    __property TNotifyEvent OnCts = { read=FOnCts,write=FOnCts };
    __property TNotifyEvent OnDsr = { read=FOnDsr,write=FOnDsr };
    __property TNotifyEvent OnRing = { read=FOnRing,write=FOnRing };
    __property TNotifyEvent OnRlsd = { read=FOnRlsd,write=FOnRlsd };
    __property TCommErrorEvent OnError = { read=FOnError,write=FOnError };
    __property TCommRxCharEvent OnRxChar = { read=FOnRxChar,write=FOnRxChar };
    __property TNotifyEvent OnRxFlag = { read=FOnRxFlag,write=FOnRxFlag };
    __property TNotifyEvent OnTxEmpty = { read=FOnTxEmpty,write=FOnTxEmpty };
    __property TCommStatEvent OnComStatEvent = { read=FCommStatEvent,
                                                 write=FCommStatEvent };
};
//---------------------------------------------------------------------------
#endif
